UNIT terreno;
(* Programa: TINS'06
 * Unidad: terreno
 * Descripcin: Implementa el terreno de juego.
 * Autor: uo Martnez <niunio at pagina.de>
 *)

INTERFACE

USES
  mingro, tren;

CONST
(* Mnemnicos para las estaciones. *)
  EST_TOP = 0; { Es un tope.  Da media vuelta. }
  EST_ABA = 1; { Est un lateral y la estacin est "abajo".  }
  EST_ARR = 2; { Est un lateral y la estacin est "arriba". }
  EST_DER = 3; { Est arriba o abajo y la estacin est a la derecha. }
  EST_IZQ = 4; { Est arriba o abajo y la estacin est a la izquierda. }


TYPE
(* ESTACION:
 *   Estacin.  Puede ser una estacin completa, con pasajeros, o un tope en
 *   el que la locomotora da media vuelta. *)
  ESTACION = RECORD
    Tipo: INTEGER;
    Tiempo: INTEGER; { Si al llegar Tiempo > 0 es que lleg a la hora. }
    Pasajeros: BOOLEAN; { Indica si hay pasajeros o no. }
  END;


CONST
(* Mnemnicos para las imgenes de las vas. *)
  VIA_VACIA =  0;
  VIA_VERT  =  1;
  VIA_HORIZ =  2;
  VIA_DERAB =  3;
  VIA_IZQAB =  4;
  VIA_IZQAR =  5;
  VIA_DERAR =  6;
  VIA_CRUCE =  7;
  VIA_TOPAR =  8;
  VIA_TOPDE =  9;
  VIA_TOPAB = 10;
  VIA_TOPIZ = 11;
  VIA_VACAS = 12;

TYPE
(* ARENA:
 *   Define el terreno de juego. *)
  ARENA = CLASS
  PRIVATE
    fTiempo: LONGWORD; { Tiempo de juego, en ciclos de juego. }
    fTiempoParada: INTEGER;
    fTamannoPuzzle: INTEGER; { 3 (3x3), 4 (4x4)  5 (5x5) }
    fCasillas: ARRAY [0 .. 25] OF BYTE;
  { Lista de imgenes de vas:
    +  1 -  2: Recta.
    +  3 -  6: Curvas.
    +  7     : Cruce de vas.
    +  8 - 11: Tope.
    + 12 - 15: Sentido nico. }
    fListaVias: ARRAY [0..15] OF MG_BITMAPptr;
    fVacioX, fVacioY: INTEGER; { Coordenadas de la casilla vaca. }
    fSuelo, fCasilla: MG_BITMAPptr;
  { Estaciones de tren.  Cada "Estacin" es la mitad.  El primer ndice indica
    el lado en orden arriba, abajo, izquierda y derecha.  El segundo indica la
    estacin de arriba a abajo o de izquierda a derecha. }
    fEstacion: ARRAY [0..3, 0..4] OF ESTACION;
    fVistaEstacion: ARRAY [0..3] OF MG_BITMAPptr; { En sentido del reloj. }
    fVistaPasajeros: ARRAY [0..3] OF MG_BITMAPptr; { Igual que estaciones. } 
  { El tren. }
    fTren: LOCOMOTORA;
  { Final del juego. }
    fFinJuego, fGana: BOOLEAN;

    FUNCTION ObtieneCasilla (x, y: INTEGER): BYTE;
    PROCEDURE PonCasilla (x, y: INTEGER; Valor: BYTE);
    FUNCTION CuentaEstaciones: INTEGER;
  PUBLIC
    CONSTRUCTOR Create (Vias: MG_BITMAPptr);
    DESTRUCTOR Destroy; OVERRIDE;

  (* PosACasilla:
   *   Obtiene una posicin en pantalla y devuelve la casilla o negativo si
   *   est fuera. *)
    PROCEDURE PosACasilla (px, py: INTEGER; VAR cx, cy: INTEGER);
  (* NuevoTerreno:
   *   Genera un nuevo terreno de juego.  Cada parmetro indica quntas
   *   secciones de cada han de haber. *)
    PROCEDURE NuevoTerreno (Tamanno, Tiempo, Velocidad, Rectas, Curvas, Cruces,
			    Muertas, Vacas: INTEGER);
  (* GiraCasilla:
   *   Gira la casilla indicada, si puede. *)
    PROCEDURE GiraCasilla (aX, aY: INTEGER; SentidoReloj: BOOLEAN);
  (* DesplazaCasillas:
   *   Desplaza la fila o la columna indicada, si puede. *)
    PROCEDURE DesplazaCasillas (aX, aY: INTEGER);
  (* Actualiza:
   *   Actualiza el estado del juego. *)
    PROCEDURE Actualiza;
  (* DibujaFondo:
   *   Dibuja la hierva del fondo. *)
    PROCEDURE DibujaFondo (aBmp: MG_BITMAPptr);
  (* Dibuja:
   *   Dibuja el terreno de juego. *)
    PROCEDURE Dibuja (aBmp: MG_BITMAPptr);

  (* Propiedades para acceder a los datos del tablero. *)
    PROPERTY Casilla[x, y: INTEGER]: BYTE READ ObtieneCasilla WRITE PonCasilla;
    PROPERTY VacioX: INTEGER READ fVacioX;
    PROPERTY VacioY: INTEGER READ fVacioY;
    PROPERTY NumEstaciones: INTEGER READ CuentaEstaciones;
    PROPERTY FinJuego: BOOLEAN READ fFinJuego;
    PROPERTY Gana: BOOLEAN READ fGana;
  END;


CONST
  FUERA = 255; { Indica que est fuera del tablero. }



IMPLEMENTATION

USES
  error, sysutils;


  
(************
 * ESTACION *
 ************)



(*********
 * ARENA *
 *********)

(* ARENA.ObtieneCasilla:
 *   Devuelve la casilla indicada por las coordenadas o Vacio si est fuera
 *   del tablero. *)
FUNCTION ARENA.ObtieneCasilla (x, y: INTEGER): BYTE;
BEGIN
{ Comprueba si est fuera del tablero. }
  IF (x < 0) OR (y < 0) OR (x >= fTamannoPuzzle) OR (y >= fTamannoPuzzle) THEN
    ObtieneCasilla := FUERA
  ELSE
    ObtieneCasilla := fCasillas[(y * 5) + x];
END;



(* ARENA.PonCasilla:
 *   Establece el valor de la casilla indicada por las coordenadas. *)
PROCEDURE ARENA.PonCasilla (x, y: INTEGER; Valor: BYTE);
BEGIN
{ Comprueba si est fuera del tablero. }
  IF (x < 0) OR (y < 0) OR (x >= fTamannoPuzzle) OR (y >= fTamannoPuzzle) THEN
    EXIT;
{ En otro caso, asigna el nuevo valor. }
  fCasillas[(y * 5) + x] := Valor;
END;



(* ARENA.CuentaEstaciones:
 *   Devuelve el nmero de estaciones que faltan por recorrer. *)
FUNCTION ARENA.CuentaEstaciones: INTEGER;
VAR 
  X, Y, Pasajeros: INTEGER;
BEGIN
  Pasajeros := 0;
  FOR X := 0 TO 3 DO FOR Y := 0 TO fTamannoPuzzle - 1 DO
  BEGIN
  { Cuenta slo las impares. }
    IF (Y MOD 2) = 1 THEN
      IF fEstacion[X, Y].Pasajeros THEN
	INC (Pasajeros);
  END;
  CuentaEstaciones := Pasajeros;
END;



(* ARENA.Create:
 *   Constructor. *)
CONSTRUCTOR ARENA.Create (Vias: MG_BITMAPptr);
VAR
  Cnt: INTEGER;
BEGIN
{ Crea el terreno. }
  INHERITED Create;
  fTamannoPuzzle := 5;
{ Carga los grficos. }
{ Vas }
  FOR Cnt := 0 TO 15 DO
  BEGIN
    IF (Cnt <> 1) AND (Cnt <> 3) AND (Cnt <> 7) AND (Cnt <> 8) THEN
    BEGIN
      fListaVias[Cnt] := mgCreateBitmap (64, 64);
      IF fListaVias[Cnt] = NIL THEN
        RAISE EXCEPTION.Create ('Can''t create tiles');
      mgClearBitmapToColor (fListaVias[Cnt], mgColor (255, 0, 255));
    END;
  END;
  fListaVias[1] := mgCreateSubBitmap (Vias, 0, 0, 64, 64);
  mgDrawRotatedSprite (fListaVias[2], fListaVias[1], 0, 0, 64);
  fListaVias[3] := mgCreateSubBitmap (Vias, 64, 0, 64, 64);
  mgDrawSpriteHflip (fListaVias[4], fListaVias[3], 0, 0);
  mgDrawSpriteVHflip (fListaVias[5], fListaVias[3], 0, 0);
  mgDrawSpriteVflip (fListaVias[6], fListaVias[3], 0, 0);
  fListaVias[7] := mgCreateSubBitmap (Vias, 128, 0, 64, 64);
  fListaVias[8] := mgCreateSubBitmap (Vias, 192, 0, 64, 64);
  mgDrawRotatedSprite (fListaVias[9], fListaVias[8], 0, 0, 64);
  mgDrawSpriteVflip (fListaVias[10], fListaVias[8], 0, 0);
  mgDrawSpriteHflip (fListaVias[11], fListaVias[9], 0, 0);
{ Estaciones }
  fVistaEstacion[0] := mgCreateSubBitmap (Vias,   0, 64, 128, 64);
  fVistaEstacion[1] := mgCreateBitmap (128, 64);
  fVistaEstacion[2] := mgCreateBitmap (64, 128);
  fVistaEstacion[3] := mgCreateSubBitmap (Vias, 128, 64, 64, 128);
  FOR Cnt := 0 TO 3 DO
    IF fVistaEstacion[Cnt] = NIL THEN
      RAISE EXCEPTION.Create ('Can''t create station');
  mgClearBitmapToColor (fVistaEstacion[1], mgColor (255, 0, 255));
  mgClearBitmapToColor (fVistaEstacion[2], mgColor (255, 0, 255));
  mgDrawSpriteVflip (fVistaEstacion[1], fVistaEstacion[0], 0, 0);
  mgDrawSpriteHflip (fVistaEstacion[2], fVistaEstacion[3], 0, 0);
{ Pasajeros }
  fVistaPasajeros[0] := mgCreateSubBitmap (Vias,   0, 128, 128, 64);
  fVistaPasajeros[1] := mgCreateBitmap (128, 64);
  fVistaPasajeros[2] := mgCreateBitmap (64, 128);
  fVistaPasajeros[3] := mgCreateSubBitmap (Vias, 192, 64, 64, 128);
  FOR Cnt := 0 TO 3 DO
    IF fVistaPasajeros[Cnt] = NIL THEN
      RAISE EXCEPTION.Create ('Can''t create station');
  mgClearBitmapToColor (fVistaPasajeros[1], mgColor (255, 0, 255));
  mgClearBitmapToColor (fVistaPasajeros[2], mgColor (255, 0, 255));
  mgDrawSpriteVflip (fVistaPasajeros[1], fVistaPasajeros[0], 0, 0);
  mgDrawSpriteHflip (fVistaPasajeros[2], fVistaPasajeros[3], 0, 0);
{ Suelo }
  fSuelo   := mgCreateSubBitmap (Vias, 128, 192, 64, 64);
  fCasilla := mgCreateSubBitmap (Vias, 192, 192, 64, 64);
  fListaVias[12] := mgCreateSubBitmap (Vias,  64, 192, 64, 64);
{ Objetos. }
  fTren := LOCOMOTORA.Create (Vias);
  IF fTren = NIL THEN
    RAISE EXCEPTION.Create ('Can''t create train!');
END;



(* ARENA.Destroy:
 *   Destructor *)
DESTRUCTOR ARENA.Destroy;
VAR
  Cnt: INTEGER;
BEGIN
{ Libera imgenes. }
  Cnt := 1;
  WHILE (Cnt < 16) AND (fListaVias[Cnt] <> NIL) DO
  BEGIN
    mgDestroyBitmap (fListaVias[Cnt]);
    INC (CNT);
  END;
  Cnt := 0;
  WHILE (Cnt < 4) AND (fVistaEstacion[Cnt] <> NIL) DO
  BEGIN
    mgDestroyBitmap (fVistaEstacion[Cnt]);
    INC (CNT);
  END;
  WHILE (Cnt < 4) AND (fVistaPasajeros[Cnt] <> NIL) DO
  BEGIN
    mgDestroyBitmap (fVistaPasajeros[Cnt]);
    INC (CNT);
  END;
  IF fSuelo   <> NIL THEN mgDestroyBitmap (fSuelo);
  IF fCasilla <> NIL THEN mgDestroyBitmap (fCasilla);
{ Libera objetos. }
  IF fTren <> NIL THEN fTren.Destroy;
{ Contina la destruccin. }
  INHERITED Destroy;
END;



(* ARENA.PosACasilla:
 *   Obtiene una posicin en pantalla y devuelve la casilla o negativo si
 *   est fuera. *)
PROCEDURE ARENA.PosACasilla (px, py: INTEGER; VAR cx, cy: INTEGER);
BEGIN
{ Pasa a coordenadas locales. }
  px := px - 64; py := py - 64;
{ Comprueba si est fuera. }
  IF (0 > px) OR (px >= fTamannoPuzzle * 64) THEN
    cx := -1
  ELSE
    cx := px DIV 64;
  IF (0 > py) OR (py >= fTamannoPuzzle * 64) THEN
    cy := -1
  ELSE
    cy := py DIV 64;
END;



(* ARENA.NuevoTerreno:
   *   Genera un nuevo terreno de juego. *)
PROCEDURE ARENA.NuevoTerreno (Tamanno, Tiempo, Velocidad, Rectas, Curvas,
			      Cruces, Muertas, Vacas: INTEGER);
VAR
  Tx, Ty: INTEGER;
BEGIN
  fTiempoParada := Tiempo;
{ Comprueba que el tamao es vlido. }
  IF (3 > Tamanno) OR (Tamanno > 5) THEN
    RAISE EXCEPTION.Create ('Bad puzzle size ('+IntToStr (Tamanno)+')');
{ Comprueba que estn las piezas precisas, ni ms ni menos. }
  IF (Tamanno * Tamanno) - 1 <> (Rectas+Curvas+Cruces+Muertas+Vacas) THEN
    RAISE EXCEPTION.Create ('Wrong number of tracks in puzzle ');
{ Lo "crea". }
  fTamannoPuzzle := Tamanno;
{ "Vaciamos" el terreno. }
  FOR Tx := 0 TO (fTamannoPuzzle - 1) DO FOR Ty := 0 TO (fTamannoPuzzle - 1) DO
    Casilla[Tx, Ty] := VIA_VACIA;
{ Colocamos una recta delante del tren. }
  Casilla[fTamannoPuzzle - 1, 0] := VIA_VERT;
  DEC (Rectas);
{ Colocamos las rectas. }
  IF Rectas > 0 THEN REPEAT
    Tx := Random (fTamannoPuzzle); Ty := Random (fTamannoPuzzle);
  { Comprobamos que la posicin est vaca. }
    IF Casilla[Tx, Ty] = VIA_VACIA THEN
    BEGIN
      Casilla[Tx, Ty] := (Random (2) + VIA_VERT);
      DEC (Rectas);
    END;
  UNTIL Rectas <= 0;
{ Colocamos las curvas. }
  IF Curvas > 0 THEN REPEAT
    Tx := Random (fTamannoPuzzle); Ty := Random (fTamannoPuzzle);
  { Comprobamos que la posicin est vaca. }
    IF Casilla[Tx, Ty] = VIA_VACIA THEN
    BEGIN
      Casilla[Tx, Ty] := (Random (4) + VIA_DERAB);
      DEC (Curvas);
    END;
  UNTIL Curvas <= 0;
{ Colocamos las cruces. }
  IF Cruces > 0 THEN REPEAT
    Tx := Random (fTamannoPuzzle); Ty := Random (fTamannoPuzzle);
  { Comprobamos que la posicin est vaca. }
    IF Casilla[Tx, Ty] = VIA_VACIA THEN
    BEGIN
      Casilla[Tx, Ty] := VIA_CRUCE;
      DEC (Cruces);
    END;
  UNTIL Cruces <= 0;
{ Colocamos las vas muertas. }
  IF Muertas > 0 THEN REPEAT
    Tx := Random (fTamannoPuzzle); Ty := Random (fTamannoPuzzle);
  { Comprobamos que la posicin est vaca. }
    IF Casilla[Tx, Ty] = VIA_VACIA THEN
    BEGIN
      Casilla[Tx, Ty] := (Random (4) + VIA_TOPAR);
      DEC (Muertas);
    END;
  UNTIL Muertas <= 0;
{ Colocamos las vacas. }
  IF Vacas > 0 THEN REPEAT
    Tx := Random (fTamannoPuzzle); Ty := Random (fTamannoPuzzle);
  { Comprobamos que la posicin est vaca. }
    IF Casilla[Tx, Ty] = VIA_VACIA THEN
    BEGIN
      Casilla[Tx, Ty] := VIA_VACAS;
      DEC (Vacas);
    END;
  UNTIL Vacas <= 0;
{ Busca la casilla vaca. }
  fVacioX := -1; fVacioY := -1;
  FOR Tx := 0 TO (fTamannoPuzzle - 1) DO FOR Ty := 0 TO (fTamannoPuzzle - 1) DO
    IF Casilla[Tx, Ty] = VIA_VACIA THEN
    BEGIN
      IF fVacioX <> -1 THEN
	RAISE EXCEPTION.Create ('Error creating puzzle!');
      fVacioX := Tx; fVacioY := Ty;
    END;
{ Coloca las estaciones. }
  FOR Tx := 0 TO fTamannoPuzzle - 1 DO
  BEGIN
    IF Tx MOD 2 = 0 THEN
    BEGIN
      fEstacion[0, Tx].Tipo := EST_DER;
      fEstacion[1, Tx].Tipo := EST_DER;
    END
    ELSE BEGIN
      fEstacion[0, Tx].Tipo := EST_IZQ;
      fEstacion[1, Tx].Tipo := EST_IZQ;
    END;
    fEstacion[0, Tx].Tiempo := 0; fEstacion[0, Tx].Pasajeros := TRUE;
    fEstacion[1, Tx].Tiempo := 0; fEstacion[1, Tx].Pasajeros := TRUE;
  END;
  FOR Ty := 0 TO fTamannoPuzzle - 1 DO
  BEGIN
    IF Ty MOD 2 = 0 THEN
    BEGIN
      fEstacion[2, Ty].Tipo := EST_ABA;
      fEstacion[3, Ty].Tipo := EST_ABA;
    END
    ELSE BEGIN
      fEstacion[2, Ty].Tipo := EST_ARR;
      fEstacion[3, Ty].Tipo := EST_ARR;
      END;
    fEstacion[2, Ty].Tiempo := 0; fEstacion[2, Ty].Pasajeros := TRUE;
    fEstacion[3, Ty].Tiempo := 0; fEstacion[3, Ty].Pasajeros := TRUE;
  END;
  IF (fTamannoPuzzle MOD 2) = 1 THEN
  BEGIN
    FOR Ty := 0 TO 3 DO
    BEGIN
      fEstacion[Ty, fTamannoPuzzle - 1].Tipo := EST_TOP;
      fEstacion[Ty, fTamannoPuzzle - 1].Pasajeros := FALSE;
    END;
  END;
{ Coloca el tren. }
  fTren.PonTren (fTamannoPuzzle - 1, -1, Velocidad);
{ Empieza el juego. }
  fTiempo := 0; fFinJuego := FALSE; fGana := FALSE;
END;



(* ARENA.GiraCasilla:
 *   Gira la casilla indicada, si puede. *)
PROCEDURE ARENA.GiraCasilla (aX, aY: INTEGER; SentidoReloj: BOOLEAN);
VAR
  Tipo: BYTE;
BEGIN
{ Comprueba que est dentro del terreno de juego. }
  IF (0 <= aX) AND (aX < fTamannoPuzzle) AND
     (0 <= aY) AND (aY < fTamannoPuzzle) THEN
  BEGIN
  { Si la casilla est ocupada por el tren, no se puede girar. }
    IF fTren.OcupaCasilla (aX, aY) THEN EXIT;
    Tipo := Casilla[aX, aY];
    IF (Tipo <= 0) OR (Tipo = 7) THEN EXIT; { No hay casilla o no gira. }
  { No todas las casillas giran igual, as que... }
    CASE Tipo OF
    1: { Tramo recto vertical. }
      Casilla[aX, aY] := 2;
    2: { Tramo recto horizontal. }
      Casilla[aX, aY] := 1;
    3 .. 6: BEGIN { Curvas. }
      IF SentidoReloj THEN
      BEGIN
	IF Tipo <> 6 THEN
	  INC (Tipo)
	ELSE
	  Tipo := 3;
      END
      ELSE IF TIPO <> 3 THEN
	DEC (Tipo)
      ELSE
        Tipo := 6;
      Casilla[aX, aY] := Tipo;
    END;
    8 .. 11: BEGIN { Topes. }
      IF SentidoReloj THEN
      BEGIN
	IF Tipo <> 11 THEN
	  INC (Tipo)
	ELSE
	  Tipo := 8;
      END
      ELSE IF TIPO <> 8 THEN
	DEC (Tipo)
      ELSE
        Tipo := 11;
      Casilla[aX, aY] := Tipo;
    END;
    END;
  END;
END;



(* ARENA.DesplazaCasillas:
 *   Desplaza la fila o la columna indicada, si puede. *)
PROCEDURE ARENA.DesplazaCasillas (aX, aY: INTEGER);
VAR
  tX, tY: INTEGER;
BEGIN
{ Comprueba que se puede desplazar la fila o la columna. }
  IF fTren.OcupaCasilla (aX, aY) OR 
     ((aX <> fVacioX) AND (aY <> fVacioY)) OR 
     ((aX = fVacioX) AND (aY = fVacioY)) THEN
  BEGIN
    EXIT;
  END;
{ Vertical u horizontal? }
  IF aY = fVacioY THEN
  BEGIN { Horizontal. }
  { A la derecha o a la izquierda? }
    IF aX < fVacioX THEN
    BEGIN { A la Izquierda. }
    { Comprueba si puede desplazar la fila. }
      FOR tX := fVacioX DOWNTO aX DO
        IF fTren.OcupaCasilla(tX, fVacioY) THEN EXIT;
    { Desplaza. }
      FOR tX := fVacioX DOWNTO aX DO
        Casilla[tX, fVacioY] := Casilla[tX - 1, fVacioY];
    END
    ELSE BEGIN { A la Derecha. }
    { Comprueba si puede desplazar la fila. }
      FOR tX := fVacioX TO aX DO
        IF fTren.OcupaCasilla(tX, fVacioY) THEN EXIT;
    { Desplaza. }
      FOR tX := fVacioX TO aX DO
        Casilla[tX, fVacioY] := Casilla[tX + 1, fVacioY];
    END;
  { Recoloca el espacio vaco. }
    fVacioX := aX;
  END
  ELSE BEGIN { Vertical. }
  { Arriba o abajo? }
    IF aY < fVacioY THEN
    BEGIN { Abajo. }
    { Comprueba si puede desplazar la columna. }
      FOR tY := fVacioY DOWNTO aY DO
        IF fTren.OcupaCasilla(fVacioX, tY) THEN EXIT;
    { Desplaza. }
      FOR tY := fVacioY DOWNTO aY DO
        Casilla[fVacioX, tY] := Casilla[fVacioX, tY - 1];
    END
    ELSE BEGIN { Arriba. }
    { Comprueba si puede desplazar la columna. }
      FOR tY := fVacioY TO aY DO
        IF fTren.OcupaCasilla(fVacioX, tY) THEN EXIT;
    { Desplaza. }
      FOR tY := fVacioY TO aY DO
        Casilla[fVacioX, tY] := Casilla[fVacioX, tY + 1];
    END;
  { Recoloca el espacio vaco. }
    fVacioY := aY;
  END;
{ Recoloca el espacio vaco. }
  Casilla[aX, aY] := 0;
END;



(* ARENA.Actualiza:
 *   Actualiza el estado del juego. *)
PROCEDURE ARENA.Actualiza;
VAR
  Via: BYTE; { Tipo de va sobre la que est el tren. }

(* Estacin o tope. *)
  PROCEDURE EstacionSuperior (Indice: INTEGER);
  BEGIN
    IF (fEstacion[0, Indice].Tipo = EST_TOP) AND (fTren.Angl = 64) THEN
      fTren.Gira (192, TRUE)
    ELSE IF (fEstacion[0, Indice].Tipo = EST_DER) THEN
    BEGIN
      IF fTren.Angl = 64 THEN
	fTren.Gira (128 ,TRUE)
      ELSE IF fTren.Angl = 0 THEN
	fTren.Gira (192, FALSE);
    END
    ELSE IF (fEstacion[0, Indice].Tipo = EST_IZQ) THEN
    BEGIN
      IF fTren.Angl = 64 THEN
	fTren.Gira (0 ,FALSE)
      ELSE IF fTren.Angl = 128 THEN
	fTren.Gira (192, TRUE);
    END
  END;
  
  PROCEDURE EstacionInferior (Indice: INTEGER);
  BEGIN
    IF (fEstacion[1, Indice].Tipo = EST_TOP) AND (fTren.Angl = 192) THEN
      fTren.Gira (64, TRUE)
    ELSE IF (fEstacion[1, Indice].Tipo = EST_DER) THEN
    BEGIN
      IF fTren.Angl = 192 THEN
	fTren.Gira (128 ,FALSE)
      ELSE IF fTren.Angl = 0 THEN
	fTren.Gira (64, TRUE);
    END
    ELSE IF (fEstacion[1, Indice].Tipo = EST_IZQ) THEN
    BEGIN
      IF fTren.Angl = 192 THEN
	fTren.Gira (0, TRUE)
      ELSE IF fTren.Angl = 128 THEN
	fTren.Gira (64, FALSE);
    END
  END;

  PROCEDURE EstacionIzquierda (Indice: INTEGER);
  BEGIN
    IF (fEstacion[2, Indice].Tipo = EST_TOP) AND (fTren.Angl = 0) THEN
      fTren.Gira (128, TRUE)
    ELSE IF (fEstacion[2, Indice].Tipo = EST_ABA) THEN
    BEGIN
      IF fTren.Angl = 0 THEN
	fTren.Gira (192, FALSE)
      ELSE IF fTren.Angl = 64 THEN
	fTren.Gira (128, TRUE);
    END
    ELSE IF (fEstacion[2, Indice].Tipo = EST_ARR) THEN
    BEGIN
      IF fTren.Angl = 0 THEN
	fTren.Gira (64, TRUE)
      ELSE IF fTren.Angl = 192 THEN
	fTren.Gira (128, FALSE);
    END
  END;
  
  PROCEDURE EstacionDerecha (Indice: INTEGER);
  BEGIN
    IF (fEstacion[3, Indice].Tipo = EST_TOP) AND (fTren.Angl = 128) THEN
    BEGIN
      fTren.Gira (0, TRUE);
    END
    ELSE IF (fEstacion[3, Indice].Tipo = EST_ABA) THEN
    BEGIN
      IF fTren.Angl = 128 THEN
	fTren.Gira (192, TRUE)
      ELSE IF fTren.Angl = 64 THEN
	fTren.Gira (0, FALSE);
    END
    ELSE IF (fEstacion[3, Indice].Tipo = EST_ARR) THEN
    BEGIN
      IF fTren.Angl = 128 THEN
	fTren.Gira (64, FALSE)
      ELSE IF fTren.Angl = 192 THEN
	fTren.Gira (0, TRUE);
    END
  END;

(* Va curva. *)
  PROCEDURE Curva;
  BEGIN
    IF fTren.Angl = 0 THEN
      CASE Via OF
      VIA_DERAB:
	fTren.Gira (192, FALSE);
      VIA_DERAR:
	fTren.Gira (64, TRUE);
      ELSE
	Via := 0;
      END;
    IF fTren.Angl = 64 THEN
      CASE Via OF
      VIA_DERAB:
	fTren.Gira (128, TRUE);
      VIA_IZQAB:
	fTren.Gira (0, FALSE);
      ELSE
	Via := 0;
      END;
    IF fTren.Angl = 128 THEN
      CASE Via OF
      VIA_IZQAR:
	fTren.Gira (64, FALSE);
      VIA_IZQAB:
	fTren.Gira (192, TRUE);
      ELSE
	Via := 0;
      END;
    IF fTren.Angl = 192 THEN
      CASE Via OF
      VIA_IZQAR:
	fTren.Gira (0, TRUE);
      VIA_DERAR:
	fTren.Gira (128, FALSE);
      ELSE
	Via := 0;
      END;
  END;

(* Va muerta. *)
  PROCEDURE Muerta;
  BEGIN
    IF (Via = VIA_TOPIZ) AND (fTren.Angl = 0) THEN
      fTren.Gira (128, TRUE);
    IF (Via = VIA_TOPAR) AND (fTren.Angl = 64) THEN
      fTren.Gira (192, TRUE);
    IF (Via = VIA_TOPDE) AND (fTren.Angl = 128) THEN
      fTren.Gira (0, TRUE);
    IF (Via = VIA_TOPAB) AND (fTren.Angl = 192) THEN
      fTren.Gira (64, TRUE);
  END;



{ PROCEDURE ARENA.Actualiza;
  VAR
    Via: BYTE; } { Tipo de va sobre la que est el tren. }
VAR
  X, Y: INTEGER;
BEGIN
  INC (fTiempo);
{ En los tiempos pares anima el tren y comprueba el final. } 
  IF (fTiempo MOD 2) = 0 THEN
  BEGIN
    fTren.Actualiza;
  { Comprueba el final del juego. }
    IF CuentaEstaciones = 0 THEN
    BEGIN
      fFinJuego := TRUE; fGana := TRUE;
    END;
    IF NOT fFinJuego THEN
    BEGIN { Comprueba descarrilamientos. }
    { Primero, debe estar dentro del terreno de juego y medio en casilla. }
      IF (((fTren.PosX MOD 64) <> 0) AND ((fTren.PosX MOD 32) = 0)) OR 
	 (((fTren.PosY MOD 64) <> 0) AND ((fTren.PosY MOD 32) = 0)) THEN
      BEGIN
	fTren.SiguienteCasilla (X, Y);
	IF (0 <= X) AND (X < fTamannoPuzzle) AND 
	   (0 <= Y) AND (Y < fTamannoPuzzle) THEN
	BEGIN
	  Via := Casilla[X, Y];
	{ Va vaca, final seguro. }
	  IF Via = VIA_VACIA THEN
	  BEGIN
	    fFinJuego := TRUE; EXIT;
	  END;
	{ Va cruce, camino abierto. }
	  IF Via = VIA_CRUCE THEN EXIT;
	{ Segn el ngulo. }
	  IF fTren.Angl = 0 THEN
	  BEGIN
	    fFinJuego := (Via <> VIA_HORIZ) AND (Via <> VIA_DERAB) AND
			 (Via <> VIA_DERAR) AND (Via <> VIA_TOPIZ);
	    EXIT;
	  END;
	  IF fTren.Angl = 64 THEN
	  BEGIN
	    fFinJuego := (Via <> VIA_VERT ) AND (Via <> VIA_DERAB) AND
			 (Via <> VIA_IZQAB) AND (Via <> VIA_TOPAR);
	    EXIT;
	  END;
	  IF fTren.Angl = 128 THEN
	  BEGIN
	    fFinJuego := (Via <> VIA_HORIZ) AND (Via <> VIA_IZQAB) AND
			 (Via <> VIA_IZQAR) AND (Via <> VIA_TOPDE);
	    EXIT;
	  END;
	  IF fTren.Angl = 192 THEN
	  BEGIN
	    fFinJuego := (Via <> VIA_VERT ) AND (Via <> VIA_DERAR) AND
			 (Via <> VIA_IZQAR) AND (Via <> VIA_TOPAB);
	    EXIT;
	  END;
	END;
      END;
    END;
  END
{ En los impares comprueba dnde se encuentra el tren. }
  ELSE BEGIN
    PosACasilla (fTren.PosX, fTren.PosY, X, Y);
    Via := Casilla[X, Y];
    IF ((fTren.PosX MOD 64) = 0) AND ((fTren.PosY MOD 64) = 0) THEN
    BEGIN { Se encuentra dentro de un cuadro. }
      IF (Via = Fuera) AND (fTren.Estado = LOCOMOTORA_AVANZA) THEN
      BEGIN
      { Comprobamos por dnde se sali. }
	IF Y < 0 THEN
	BEGIN
	  IF fTren.PosY < 64 THEN
	    EstacionSuperior (X)
	  ELSE
	    EstacionInferior (X);
	END
	ELSE BEGIN
	  IF fTren.PosX < 64 THEN
	    EstacionIzquierda (Y)
	  ELSE
	    EstacionDerecha (Y);
	END;
      END;
      IF (VIA_DERAB <= Via) AND (Via <= VIA_DERAR) THEN
	Curva;
      IF (VIA_TOPAR <= Via) AND (Via <= VIA_TOPIZ) THEN
	Muerta;
    END
    ELSE IF (Via = Fuera) AND (fTren.Estado = LOCOMOTORA_AVANZA) THEN
    BEGIN { Se encuentra fuera, por lo que puede estar en estacin. }
    { Comprobamos por dnde se sali. }
      IF Y < 0 THEN
      BEGIN
	IF ((fTren.PosX MOD 32) = 0) AND ((fTren.PosX MOD 64) <> 0) THEN
	BEGIN
	{ Hay gente esperando el tren? }
	  IF (fTren.PosY < 64) AND (fEstacion[0, X].Pasajeros) THEN
	  BEGIN
	    fEstacion[0, X].Pasajeros := FALSE;
	    fEstacion[0, X + 1].Pasajeros := FALSE;
	    fTren.Para (fTiempoParada);
	  END
	  ELSE IF (fTren.PosY > 64) AND fEstacion[1, X].Pasajeros THEN
	  BEGIN
	    fEstacion[1, X].Pasajeros := FALSE;
	    fEstacion[1, X + 1].Pasajeros := FALSE;
	    fTren.Para (fTiempoParada);
	  END;
	END;
      END
      ELSE BEGIN
	IF ((fTren.PosY MOD 32) = 0) AND ((fTren.PosY MOD 64) <> 0) THEN
	BEGIN
	{ Hay gente esperando el tren? }
	  IF (fTren.PosX < 64) AND (fEstacion[2, Y].Pasajeros) THEN
	  BEGIN
	    fEstacion[2, Y].Pasajeros := FALSE;
	    fEstacion[2, Y + 1].Pasajeros := FALSE;
	    fTren.Para (fTiempoParada);
	  END
	  ELSE IF (fTren.PosX > 64) AND fEstacion[3, Y].Pasajeros THEN
	  BEGIN
	    fEstacion[3, Y].Pasajeros := FALSE;
	    fEstacion[3, Y + 1].Pasajeros := FALSE;
	    fTren.Para (fTiempoParada);
	  END;
	END;
      END;
    END;
  END;
END;



(* ARENA.DibujaFondo:
 *   Dibuja la hierva del fondo. *)
PROCEDURE ARENA.DibujaFondo (aBmp: MG_BITMAPptr);
VAR
  X, Y: INTEGER;
BEGIN
  FOR X := 0 TO 64 DO FOR Y := 0 TO 8 DO
    mgBlit (fSuelo, aBmp, X * 64, Y * 64);
END;



(* ARENA.Dibuja:
 *   Dibuja el terreno de juego. *)
PROCEDURE ARENA.Dibuja (aBmp: MG_BITMAPptr);
VAR
  Tx, Ty: INTEGER;
BEGIN
  FOR Tx := 0 TO (fTamannoPuzzle - 1) DO
  BEGIN
  { Dibujamos una estacin? }
    IF (Tx MOD 2) = 0 THEN
      IF Tx = (fTamannoPuzzle - 1) THEN
      BEGIN 
        mgBlit (fSuelo, aBmp, 64 + (Tx * 64), 0);
	mgDrawSprite (aBmp, fListaVias[VIA_TOPAR], 64 + Tx * 64, 0);
        mgBlit (fSuelo, aBmp, 64 + (Tx * 64), (fTamannoPuzzle + 1) * 64);
	mgDrawSprite (aBmp, fListaVias[VIA_TOPAB], 64 + (Tx * 64), (fTamannoPuzzle + 1) * 64);
      END
      ELSE BEGIN
        mgBlit (fSuelo, aBmp,  64 + (Tx * 64), 0);
        mgBlit (fSuelo, aBmp, 128 + (Tx * 64), 0);
	mgDrawSprite (aBmp, fVistaEstacion[0], 64 + Tx * 64, 0);
	IF fEstacion[0, Tx].Pasajeros THEN
	  mgDrawSprite (aBmp, fVistaPasajeros[0], 64 + Tx * 64, 0);
        mgBlit (fSuelo, aBmp,  64 + (Tx * 64), (fTamannoPuzzle + 1) * 64);
        mgBlit (fSuelo, aBmp, 128 + (Tx * 64), (fTamannoPuzzle + 1) * 64);
	mgDrawSprite (aBmp, fVistaEstacion[1], 64 + Tx * 64, (fTamannoPuzzle + 1) * 64);
	IF fEstacion[1, Tx].Pasajeros THEN
	  mgDrawSprite (aBmp, fVistaPasajeros[1], 64 + Tx * 64, (fTamannoPuzzle + 1) * 64);
      END;
    FOR Ty := 0 TO (fTamannoPuzzle - 1) DO
    BEGIN
    {Dibujamos la casilla. }
      mgBlit (fCasilla, aBmp, (Tx * 64) + 64, (Ty * 64) + 64);
      mgDrawSprite (aBmp, fListaVias[SHORTINT (Casilla[Tx, Ty])],
		    (Tx * 64) + 64, (Ty * 64) + 64);
    { Dibujamos una estacin. }		    
      IF (Ty MOD 2) = 0 THEN
	IF Ty = (fTamannoPuzzle - 1) THEN
	BEGIN
	  mgBlit (fSuelo, aBmp, 0, 64 + (Ty * 64));
	  mgDrawSprite (aBmp, fListaVias[VIA_TOPIZ], 0, 64 + Ty * 64);
	  mgBlit (fSuelo, aBmp, (fTamannoPuzzle + 1) * 64, 64 + (Ty * 64));
	  mgDrawSprite (aBmp, fListaVias[VIA_TOPDE], (fTamannoPuzzle + 1) * 64, 64 + (Ty * 64));
      END
      ELSE BEGIN
        mgBlit (fSuelo, aBmp, 0, 64 + (Ty * 64));
        mgBlit (fSuelo, aBmp, 0, 128 + (Ty * 64));
	mgDrawSprite (aBmp, fVistaEstacion[2], 0, 64 + Ty * 64);
	IF fEstacion[2, Ty].Pasajeros THEN
	  mgDrawSprite (aBmp, fVistaPasajeros[2], 0, 64 + Ty * 64);
        mgBlit (fSuelo, aBmp,  (fTamannoPuzzle + 1) * 64, 64 + (Ty * 64));
        mgBlit (fSuelo, aBmp, (fTamannoPuzzle + 1) * 64, 128 + (Ty * 64));
	mgDrawSprite (aBmp, fVistaEstacion[3], (fTamannoPuzzle + 1) * 64, 64 + (Ty * 64));
	IF fEstacion[3, Ty].Pasajeros THEN
	  mgDrawSprite (aBmp, fVistaPasajeros[3], (fTamannoPuzzle + 1) * 64, 64 + (Ty * 64));
      END;
    END;
  END;
  fTren.Dibuja (aBmp);
END;



END.

